#############
# $Id: VersionComparison.pm 15261 2012-02-29 13:11:34Z dj3 $
# Created By: ajb
# Last Maintained By: $Author: dj3 $
# Created On: 2010-11-26
# Last Changed On: $Date: 2012-02-29 13:11:34 +0000 (Wed, 29 Feb 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/lib/npg_common/VersionComparison.pm $

package npg_common::VersionComparison;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 15261 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $NEWER => 1;
Readonly::Scalar our $EQUAL => 0;
Readonly::Scalar our $OLDER => -1;

use Exporter;

our @ISA    = qw{Exporter}; ## no critic (ClassHierarchies::ProhibitExplicitISA)
our @EXPORT = qw{compare_version}; ## no critic (Modules::ProhibitAutomaticExportation)

## no critic (Documentation::RequirePodAtEnd)
=head1 NAME

npg_common::VersionComparison

=head1 VERSION

$Revision: 15261 $

=head1 SYNOPSIS

  use VersionComparison;

  my $version_comparison = compare_version( $sOrigVersion, $sNewVersion, $sDelimiter );

=head1 DESCRIPTION

This exports 1 method by default, compare_version, which you can use to compare two version strings.

=head1 SUBROUTINES/METHODS

=head2 compare_version

Compare 2 version strings, and obtain 1 if the second version is more recent (higher valued) than the first,
0 if the same or -1 if the second version is older (lower valued).

An optional delimter can be added, but defaults to a period (.)

  my $version_comparison = compare_version( $sOrigVersion, $sNewVersion, $sDelimiter );

  my $version_comparison = compare_version( q{1.2.3}, q{1.5.60} );

if the number of version components is unequal, then it will 'fill in' 0's until it gets a result, or runs out
of components.

  compare_version( q{1.2.3}, q{1.5} );

=cut

sub compare_version {
  my ( $orig_version, $new_version, $delimiter ) = @_;

  # we may as well get out asap if the strings themselves are identical
  if ( $orig_version eq $new_version ) {
    return $EQUAL;
  }

  if (! defined $delimiter ) {
    $delimiter = q{.};
  }

  $delimiter = q{[} . $delimiter . q{]};

  # check the number of components are equal
  my @orig_version_components = split /$delimiter/xms, $orig_version;
  my @new_version_components  = split /$delimiter/xms, $new_version;

  my $number_of_components = scalar @orig_version_components;
  if ( $number_of_components < scalar @new_version_components ) {
    $number_of_components = scalar @new_version_components;
  }

  my $return_value = $EQUAL;

  $number_of_components--;

  foreach my $i ( 0..$number_of_components ) {
    $orig_version_components[$i] ||= 0;
    $new_version_components[$i]  ||= 0;
    next if ( $orig_version_components[$i] == $new_version_components[$i] );

    if ( $orig_version_components[$i] < $new_version_components[$i] ) {
      $return_value = $NEWER;
    } else {
      $return_value = $OLDER;
    };
    last;
  }

  return $return_value;
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item Exporter

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: dj3 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
