#############
# $Id: pod_usage.pm 13436 2011-06-23 11:24:01Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2009-10-27
# Last Changed On: $Date: 2011-06-23 12:24:01 +0100 (Thu, 23 Jun 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/lib/npg_common/pod_usage.pm $

package npg_common::pod_usage;
use strict;
use warnings;
use Moose;
use English qw{-no_match_vars};

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13436 $ =~ /(\d+)/mxs; $r; };

has q{file_name} => (isa => q{Str}, is => q{ro}, lazy_build => 1);

# private builder to give default file_name of the program which ran if no file_name provided
sub _build_file_name {
  my ($self) = @_;
  return $PROGRAM_NAME;
}


# main method to use
sub usage_sections {
  my ($self) = @_;
  my $file_name = $self->file_name();

  ##no critic (ProhibitBacktickOperators)
  my $text = `pod2text $file_name`;
  if ($CHILD_ERROR != 0) {
    return q{};
  }
  ## use critic
  my @lines = split /\n/xms, $text;
  my @blocks = ();
  my $temp_block = q{};
  foreach my $line (@lines) {
    if ($line !~ /\A\s+/xms && $line ne q{}) {
      if ($temp_block) {
        push @blocks, $temp_block;
      }
      $temp_block = q{};
    }
    $temp_block .= $line . qq{\n};
  }

  my $return_string = q{};
  foreach my $block (@blocks) {
    if ($block =~ /\AUSAGE/xms
         ||
        $block =~ /\ASYNOPSIS/xms
         ||
        $block =~ /\AREQUIRED[ ]ARGUMENTS/xms) {
          $return_string .= $block;
        }
  }

  return $return_string;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_common::pod_usage

=head1 VERSION

$LastChangedRevision: 13436 $

=head1 SYNOPSIS

Typically, you will just want to output the usage sections directly

  print npg_common::pod_usage->new()->usage_sections(); # runs on the current program file by default
  print npg_common::pod_usage->new({file_name => $sFileName})->usage_sections();

although you can create an object and capture the usage_sections into a string variable

  my $oPodUsage = npg_common::pod_usage->new();
  my $oPodUsage = npg_common::pod_usage->new({file_name => $sFileName});

=head1 DESCRIPTION

This object module runs pod2text on a file, and outputs the usage sections (SYNOPSIS, USAGE and REQUIRED ARGUMENTS)
so that you could use these in a help/usage output. An empty string is returned if none of these sections are found
in the POD or pod2text is not found in your path

=head1 SUBROUTINES/METHODS

=head2 usage_sections - the only exported function, which runs pod2text and returns a string containing the usage sections

  my $sUsageSections = $oPodUsage->usage_sections();

=head2 file_name

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
