#############
# $Id: read_small_file.pm 9166 2010-04-29 14:33:49Z mg8 $
# Created By: ajb
# Mast Maintained By: $Author: mg8 $
# Created On: 2009-10-08
# Last Changed On: $Date: 2010-04-29 15:33:49 +0100 (Thu, 29 Apr 2010) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/lib/npg_common/roles/read_small_file.pm $

package npg_common::roles::read_small_file;
use strict;   # here to satisfy
use warnings; # webpublish
use Moose::Role;
use Carp;
use Perl6::Slurp;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 9166 $ =~ /(\d+)/mxs; $r; };

has q{filename}       => ( isa => q{Str}, is => q{ro}, required => 1 );
has q{file_contents}  => ( isa => q{Any}, is => q{ro}, lazy_build => 1);

sub _build_file_contents {
  my ($self) = @_;
  if ($self->filename() =~ /jpg|gif|jpeg|tiff|xl/xms) {
    carp $self->filename() . q{ appears to be an image or Excel file. Currently can't be processed.};
    return;
  }
  my $contents = slurp($self->filename());
  return $contents;
}

1;
__END__

=head1 NAME

npg_common::roles::read_small_file

=head1 VERSION

$LastChangedRevision: 9166 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_common::roles::read_small_file};

  my $MyPackage = MyPackage->new({
    filename => $sFileName,
  });

=head1 DESCRIPTION

This role imports the features which enable the reading of small text files into memory.

=head1 SUBROUTINES/METHODS

=head2 filename - required attribute

=head2 file_contents - returns the contents of the file

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item Readonly

=item Perl6::Slurp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
