#########
# Author:        gq1
# Maintainer:    $Author: mg8 $
# Created:       2010-05-05
# Last Modified: $Date: 2010-06-17 11:43:20 +0100 (Thu, 17 Jun 2010) $
# Id:            $Id: folder_validation.pm 9638 2010-06-17 10:43:20Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/lib/npg_common/roles/run/folder_validation.pm $
#

package npg_common::roles::run::folder_validation;
use Moose::Role;
use strict;
use warnings;
use Carp qw(carp cluck croak confess);
use Readonly;
use npg::api::run;
use npg::api::util;
use English qw{-no_match_vars};

with qw{npg_common::roles::run::short_info};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 9638 $ =~ /(\d+)/mxs; $r; };

has 'no_npg_check'=>  ( isa            => q{Bool},
                         is            => q{rw},
                         documentation => q{option to stop checking run_folder from npg},
                         default       => 0,
                       );

has 'npg_api_run' =>  ( isa => q{npg::api::run},
                        is => q{rw},
                        lazy_build => 1,
                        documentation => 'npg api run object',
                       );

sub _build_npg_api_run {
  my $self = shift;

  my $id_run = $self->id_run();
  my $util   = $self->npg_api_util();
  return npg::api::run->new({id_run => $id_run,
                              util   => $util,
                             });
}

has 'npg_api_util' =>  ( isa => q{npg::api::util},
                        is => q{rw},
                        lazy_build => 1,
                        documentation => 'npg api util object',
                       );

sub _build_npg_api_util {
  my $self = shift;

  return npg::api::util->new();
}

sub check{
  my $self = shift;

  my $run_folder = $self->run_folder();
  if($self->no_npg_check){
    carp "Run folder $run_folder will not be checked by NPG";
    return 1;
  }

  my $run_folder_npg;

  eval{
    $run_folder_npg = $self->npg_api_run->run_folder();
    1;
  } or do {
    carp $EVAL_ERROR;
    return;
  };

  if(! ($run_folder eq $run_folder_npg )){
    carp "Run folder $run_folder does not match $run_folder_npg from NPG";
    return;
  }

  return 1;
}

1;
__END__

=head1 NAME

npg_common::roles::run::folder_validation

=head1 VERSION

$LastChangedRevision: 9638 $

=head1 SYNOPSIS

use Moose;
with qw{npg_common::roles::run::folder_validation};

This requires short_info role, which requires either run_folder given in construction of object or having _build_run_folder lazy build method

$validationObj->check();

=head1 DESCRIPTION

Given a run_folder, and get the final digits of it to be the id_run, check this is genuine or not in NPG

=head1 SUBROUTINES/METHODS

=head2 check - compare the given run_folder and the run_folder from npg api

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item Readonly

=item npg_common::roles::run::short_info

=item  npg::api::run

=item  npg::api::util

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
