#############
# Created By: Marina Gourtovaia
# Maintainer: $Author: mg8 $
# Created On: 23 April 2010
# Last Modified: $Date: 2010-09-09 10:34:54 +0100 (Thu, 09 Sep 2010) $
# $Id: tag.pm 10802 2010-09-09 09:34:54Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/lib/npg_common/roles/run/lane/tag.pm $

package npg_common::roles::run::lane::tag;

use Moose::Role;
use Moose::Util::TypeConstraints;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 10802 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $TAG_INDEX_MIN  => 0;
Readonly::Scalar our $TAG_INDEX_MAX  => 999;
Readonly::Scalar our $TAG_DELIM      => q[#];

subtype 'NPG_TagIndex'
      => as Int
      => where { $_ >= $TAG_INDEX_MIN && $_ <= $TAG_INDEX_MAX };

has 'tag_index'   => (isa        => 'Maybe[NPG_TagIndex]',
                      is         => 'rw',
                      predicate  => 'has_tag_index',
                      required   => 0,
                     );

sub tag_label {
    my $self = shift;
    my $tag_label = defined $self->tag_index ? $TAG_DELIM . $self->tag_index : q[];
    return $tag_label;
}

1;

__END__

=head1 NAME

npg_common::roles::run::lane::tag

=head1 VERSION

$LastChangedRevision: 10802 $

=head1 SYNOPSIS

=head1 DESCRIPTION

tag interface

=head1 SUBROUTINES/METHODS

=head2 tag_index - Tag index. An integer from 0 to 10000. Zero means that no tag has been matched.

=head2 tag_label Tag label as found in file names

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Moose::Util::TypeConstraints

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Marina Gourtovaia

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
