#########
# Author:        gq1
# Maintainer:    $Author: dj3 $
# Created:       2009-08-27
# Last Modified: $Date: 2012-02-29 13:11:34 +0000 (Wed, 29 Feb 2012) $
# Id:            $Id: 10-irods-Loader.t 15261 2012-02-29 13:11:34Z dj3 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/t/10-irods-Loader.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 20;

use_ok('npg_common::irods::Loader');

{

  my $loader = npg_common::irods::Loader->new(file => 'test/test.file', resource => 'seq');
  isa_ok($loader, 'npg_common::irods::Loader', 'object test');
  is($loader->new_filename, 'test.file', 'destination file name');
  
  is($loader-> _get_file_md5('t/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/5169_1.bam'), '3ecff85807c7dc73f39f30b3b2a8d6a6', 'correct md5 from corresponding md5 file');
   is($loader-> _get_file_md5('t/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/5169_3.bam'), 'd1e99fdd20aab765f1d8868cb226d02e', 'correct md5 from file itself');
   
  is( $loader->_rm_metadata("test.file", "test_att") , "rmw -d test.file test_att %", "correct rmw imeta command");
  
  is( $loader->_check_add_meta("test.file", "test_att", q{test" value}, {}), q{add -d test.file test_att 'test" value'}, 'correct add command with double quote values');
  is( $loader->_check_add_meta("test.file", "test_att", q{test' value'}, {}), q{add -d test.file test_att "test' value'"}, 'correct add command with single quote values');
  is( $loader->_check_add_meta("test.file", "test_att", q{test" value'}, {}), q{add -d test.file test_att "test' value'"}, 'correct add command with mixed quotation values');
  
    my ($rmw_cmd, $add_cmd) = $loader->_check_add_meta("test.file", "test_att", qq{test" value'}, { test_att => {test => 1}, } );
    is($rmw_cmd, qq{rmw -d test.file test_att %}, 'correct add command with mixed quotation values');
    is($add_cmd, qq{add -d test.file test_att "test' value'"}, 'correct add command with mixed quotation values');
    
  #print $loader->_get_irods_md5('testfile'), "---\n";;
  #$loader->_check_meta_data('/seq/5169/5169_1#0.bam');
  #$loader->_get_file_md5('archive/lane1/5169_1#0.bam');
  #print join "\n", keys %{$loader->get_collection_file_list('/seq/5274')}, "\n";  
  #print @{$loader->imeta_query( {type=>'bas'} )}, "\n";
  
}

#{   
#  my $loader = npg_common::irods::Loader->new(file      => 'test2',
#                                               meta_data => {test_att1 => q{v1's values}, test_att2 => q{v2"s values}, }, 
#                                              unique_meta_data => {test_att1 => 1, test_att2=>1}
#                                             );                                               
#  $loader->add_meta();
#}

{
	my $loader = new npg_common::irods::Loader(file => 't/data/staging/120118_HS22_07385_A_D0GKHACXX/runParameters.xml');
	my $r = 0;
	eval { $r = $loader->run(); };
	my $err = $@;
	warn $err if $err;
	is($err,'');
	is($r,1);
}

{
	# we expect it to fail this time
	my $loader = new npg_common::irods::Loader(file => 't/data/staging/120118_HS22_07385_A_D0GKHACXX/runParameters.xml');
	my $r = 0;
	eval { $r = $loader->run(); };
	my $err = $@;
	warn $err if $err;
	is($err,'');
	isnt($r,1);
}

{
	my $loader = new npg_common::irods::Loader(file => 'xyzzy');
	is($loader->rm_file('runParameters.xml'),1);
}

# Try adding a directory
{
	my $loader = new npg_common::irods::Loader(file => 't/data/staging/120118_HS22_07385_A_D0GKHACXX/InterOp');
	my $r = 0;
	eval { $r = $loader->run(); };
	my $err = $@;
	warn $err if $err;
	is($err,'');
	is($r,1);
}

# then read the resulting collection
{
	my $loader = new npg_common::irods::Loader();
	my $col_list = $loader->get_collection_file_list('InterOp');
	is(scalar keys %$col_list, 7);
}

# then delete the collection
{
	my $loader = new npg_common::irods::Loader();
	is($loader->rm_file('InterOp'),1);
}

1;
__END__
