#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-06-21
# Last Modified: $Date: 2012-02-02 10:54:00 +0000 (Thu, 02 Feb 2012) $
# Id:            $Id: 10-sequence-BAM_MarkDuplicate.t 15093 2012-02-02 10:54:00Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/t/10-sequence-BAM_MarkDuplicate.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 19;
use Test::Exception;
use Cwd;

use lib q{t};
use t::util;

my $util = t::util->new();
my $temp_dir = $util->temp_directory();

use_ok('npg_common::sequence::BAM_MarkDuplicate');

{
  my $picard_jar_file = 't/bin/aligners/picard/current/MarkDuplicates.jar';

  my $bam = npg_common::sequence::BAM_MarkDuplicate->new(
               {
                 input_bam     => 'input.bam',
                 output_bam    => 'output.bam',
                 metrics_json  => 'metrics.json',
                 jar_file      => 't/bin/aligners/picard/current/MarkDuplicates.jar',
                 not_strip_bam_tag => 1,
               });

  isa_ok($bam, 'npg_common::sequence::BAM_MarkDuplicate', 'object test');
  
  lives_ok {$bam->temp_dir} 'temp dir generated';
  
  lives_ok {$bam->metrics_file()} 'temp metrics file';
  
  lives_ok {$bam->_result} 'result object';

  $bam->metrics_file('metrics.txt');

  $bam->temp_dir($temp_dir);

  my $abs_jar_file = getcwd().q{/t/bin/aligners/picard/picard-tools-1.31/MarkDuplicates.jar};

  is($bam->mark_duplicate_cmd(), qq{/software/bin/java -Xmx3000m -jar $abs_jar_file INPUT=input.bam OUTPUT=output.bam METRICS_FILE=metrics.txt ASSUME_SORTED='true' CREATE_INDEX='TRUE' CREATE_MD5_FILE='TRUE' MAX_FILE_HANDLES_FOR_READ_ENDS_MAP='900' READ_NAME_REGEX='[a-zA-Z0-9_]+:[0-9]:([0-9]+):([0-9]+):([0-9]+).*' REMOVE_DUPLICATES='false' VALIDATION_STRINGENCY='SILENT' VERBOSITY='ERROR'}, 'correct picard command');
  
  like ( $bam->samtools_sort_cmd(), qr{^\/software\/solexa\/bin\/aligners\/samtools/samtools\-[\d\.]+\/samtools[ ]sort[ ]input.bam[ ]$temp_dir\/sorted$}, 'correct samtools sort command');

}

{

  my $bam = npg_common::sequence::BAM_MarkDuplicate->new(
               {
                 input_bam     => 't/data/sequence/SecondCall/4392_1.bam',
                 output_bam    => 'output.bam',
                 metrics_json  => 'metrics.json',
                 sort_input    => 1,
                 temp_dir      => $temp_dir,
                 metrics_file  => $temp_dir . '/metrics.txt',
                 change_bam_header => 1,
                 not_strip_bam_tag => 1,
               });
               
  my $samtools_cmd = $bam->samtools_cmd();
  SKIP: {
      skip 'because samtools not found', 11 if !-e $samtools_cmd;

      like( $samtools_cmd, qr{^/software/solexa/bin/aligners/samtools\S+}, 'default samtools command returned');
  
      my $markduplicates_jar = $bam->jar_file();
      my $expect_mark_duplicate_cmd = qq{/software/bin/java -Xmx3000m -jar $markduplicates_jar INPUT=${temp_dir}/sorted.bam OUTPUT=/dev/stdout METRICS_FILE=${temp_dir}/metrics.txt ASSUME_SORTED='true' COMPRESSION_LEVEL='0' MAX_FILE_HANDLES_FOR_READ_ENDS_MAP='900' READ_NAME_REGEX='[a-zA-Z0-9_]+:[0-9]:([0-9]+):([0-9]+):([0-9]+).*' REMOVE_DUPLICATES='false' VALIDATION_STRINGENCY='SILENT' VERBOSITY='ERROR'};
      is($bam->mark_duplicate_cmd(), $expect_mark_duplicate_cmd, 'correct picard command');

      ok( $bam->no_alignment(), 'input bam with alignment');

      my $change_bam_header_jar = $bam->change_bam_header_jar_file();
      like($change_bam_header_jar, qr{^\/software\/solexa\/bin\/aligners\/illumina2bam\/illumina2bam\/dist\/ChangeBamHeader\.jar$}, 'ChangeBamHeader jar with abs path');
      
      my $bam_tag_stripper_jar = $bam->bam_tag_stripper_jar_file();
      like($bam_tag_stripper_jar, qr{^\/software\/solexa\/bin\/aligners\/illumina2bam\/illumina2bam\/dist\/BamTagStripper\.jar$}, 'BamTagStripper jar with abs path');
     
      is($bam->bam_tag_stripper_cmd(), qq{/software/bin/java -Xmx1000m -jar $bam_tag_stripper_jar INPUT=t/data/sequence/SecondCall/4392_1.bam OUTPUT=output.bam TMP_DIR=$temp_dir CREATE_INDEX='TRUE' CREATE_MD5_FILE='TRUE' VALIDATION_STRINGENCY='SILENT' VERBOSITY='INFO' STRIP='OQ'}, 'bam_tag_stripper command');
      
      $bam->clear_bam_tag_stripper_cmd();  
      $bam->clear_no_alignment();

      $bam->input_bam('t/data/sequence/6062_1#0.bam');
      ok( !$bam->no_alignment(), 'input bam without alignment');
      
      my $samtools_sort_pg = $bam->_samtools_sort_pg();
      like ($samtools_sort_pg, qr{^ID\:samtools_sort;PN:samtools;VN\:.+;CL\:}, 'correct samtools sort pg');
      
      my $markduplicate_pg = $bam->_mark_duplicate_pg();
      is($markduplicate_pg, "ID:picard_markduplicates;PN:Picard;VN:1.47;CL:$expect_mark_duplicate_cmd",'correct mark duplicates pg');

      is($bam->change_bam_header_cmd(), qq{/software/bin/java -Xmx1000m -jar $change_bam_header_jar INPUT=/dev/stdin OUTPUT=output.bam TMP_DIR=$temp_dir CREATE_INDEX='TRUE' CREATE_MD5_FILE='TRUE' VALIDATION_STRINGENCY='SILENT' VERBOSITY='INFO' PG="${samtools_sort_pg}" PG="${markduplicate_pg}"}, 'change bam header command');
      
      $bam->clear_change_bam_header_cmd();
      $bam->not_strip_bam_tag(0);

      is($bam->change_bam_header_cmd(), qq{/software/bin/java -Xmx1000m -jar $change_bam_header_jar INPUT=/dev/stdin OUTPUT=/dev/stdout TMP_DIR=$temp_dir COMPRESSION_LEVEL='0' VALIDATION_STRINGENCY='SILENT' VERBOSITY='INFO' PG="${samtools_sort_pg}" PG="${markduplicate_pg}"}, 'change bam header command');

      is($bam->bam_tag_stripper_cmd(), qq{/software/bin/java -Xmx1000m -jar $bam_tag_stripper_jar INPUT=/dev/stdin OUTPUT=output.bam TMP_DIR=$temp_dir CREATE_INDEX='TRUE' CREATE_MD5_FILE='TRUE' VALIDATION_STRINGENCY='SILENT' VERBOSITY='INFO' STRIP='OQ'}, 'bam_tag_stripper command');
      
     }

}

1;
