#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2010-10-29
# Last Modified: $Date: 2010-11-04 10:01:48 +0000 (Thu, 04 Nov 2010) $
# Id:            $Id: 10-sequence-SAM_Fastq_Check.t 11596 2010-11-04 10:01:48Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/t/10-sequence-SAM_Fastq_Check.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 11;
use Test::Exception;

use_ok('npg_common::sequence::SAM_Fastq_Check');

my $fastq1    = 't/data/sequence/5390/5390_1_1#0.fastq';
my $fastq2    = 't/data/sequence/5390/5390_1_2#0.fastq';
my $sam_file  = 't/data/sequence/5390/5390_1#0.sam';
my $sam_file_single = 't/data/sequence/5390/5390_1#0_single.sam';

{
  my $sam = npg_common::sequence::SAM_Fastq_Check->new({
                 fastq_files        => [$fastq1],
               });
               
  isa_ok($sam, 'npg_common::sequence::SAM_Fastq_Check', 'object test');
  
  open my $fastq_fh1, '<', $fastq1;
  open my $fastq_fh2, '<', $fastq2;
  my @fastq_fhs;
  push @fastq_fhs, $fastq_fh1;
  push @fastq_fhs, $fastq_fh2;

  my $sam_line = q{IL34_5390:1:1:12222:999#0	99	gi|220673408|emb|FM211187.1|	1557268	60	54M	=	1557428	214	NATTTAAGTTTCATGGCACCAAAACCATCAATCTTACAGTCGATATTGTGGTCG	'3/576A@@@@@@@@@;@@@@@@@@@@@@@@@>@@@@@>@@@@@@@@@@@@@@@	XT:A:U	NM:i:2	SM:i:37	AM:i:37	X0:i:1	X1:i:0	XM:i:2	XO:i:0	XG:i:0	MD:Z:0G49A3};
  ok( $sam->_check_sam_line_with_fastq($sam_line ,\@fastq_fhs), 'forward bases and qualities in sam line match the values in fastq');
 
  $sam_line = q{IL34_5390:1:1:12222:999#0	147	gi|220673408|emb|FM211187.1|	1557428	60	54M	=	1557268	-214	ATTTATTTCCGTTGGCATCGATAGCGACAAGACCTTCTTCTACTTCTGCAACT	BB?B?????B?????B??BBBBBBBB?BBBB?BBBBBBBBBBBBBBBBBBBBBB	XT:A:U	NM:i:0	SM:i:37	AM:i:37	X0:i:1	X1:i:0	XM:i:0	XO:i:0	XG:i:0	MD:Z:54};
  ok( !$sam->_check_sam_line_with_fastq($sam_line ,\@fastq_fhs), 'bases in sam line match the values in fastq');
 
 $sam_line = q{IL34_5390:1:1:13646:1008#0	83	gi|220673408|emb|FM211187.1|	215513	60	54M	=	215317	-250	TACGAAAGTATCTTCGATTCTTTCTGTCTTGCGTGCTAATTTAGTAAATCATTN	@@@@@@@@@@@@@@@@@@@@@@>@@@@@@@@@:@@@@@@@@@@@@@@@@@@@@(	XT:A:U	NM:i:1	SM:i:37	AM:i:37	X0:i:1	X1:i:0	XM:i:1	XO:i:0	XG:i:0	MD:Z:53T0};
  ok( $sam->_check_sam_line_with_fastq($sam_line ,\@fastq_fhs), 'reverse bases and qualities in sam line match the values in fastq');
  
   $sam_line = q{IL34_5390:1:1:13646:1008#0	163	gi|220673408|emb|FM211187.1|	215317	60	54M	=	215513	250	AGCATCAATGGTTAGACATGCTTACAGAGGATGACTTTAAAGAATTGACCAAAG	BBBBBBBBBBBBBBBBBBB?BBBB?B?BB??????B????B?B????????B?	XT:A:U	NM:i:0	SM:i:37	AM:i:37	X0:i:1	X1:i:0	XM:i:0	XO:i:0	XG:i:0	MD:Z:54};
  ok( !$sam->_check_sam_line_with_fastq($sam_line ,\@fastq_fhs), 'qualities in sam line not match the values in fastq');
  
  throws_ok {$sam->_check_sam_line_with_fastq($sam_line ,\@fastq_fhs)}  qr{The order of reads in sam and fastq file not match}, 'the order of reads is wrong';
}

{

  my $sam = npg_common::sequence::SAM_Fastq_Check->new({
                 sam                => $sam_file_single,
                 fastq_files        => [$fastq1],
               });
               
  isa_ok($sam, 'npg_common::sequence::SAM_Fastq_Check', 'object test');

  lives_ok{$sam->process();} q{processing ok};
}

{

  my $sam = npg_common::sequence::SAM_Fastq_Check->new({
                 sam                => $sam_file,
                 fastq_files        => [$fastq1, $fastq2],
               });
               
  isa_ok($sam, 'npg_common::sequence::SAM_Fastq_Check', 'object test');

  lives_ok{$sam->process();} q{processing ok};
}


1;
