#########
# Author:        John O'Brien
# Maintainer:    $Author: mg8 $
# Created:       12/01/2011
# Last Modified: $Date: 2011-03-15 09:28:13 +0000 (Tue, 15 Mar 2011) $
# Id:            $Id: 40-roles-aligners.t 12797 2011-03-15 09:28:13Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/t/40-roles-aligners.t $
#

use strict;
use warnings;
use Moose::Util;
use Moose::Meta::Class;
use Test::More tests => 29;
use Test::Deep;
use Test::Exception;
use Test::Warn;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 12797 $ =~ /(\d+)/msx; $r; };

my $test;


# Test default attributes.
{
    lives_ok {
                $test = Moose::Meta::Class->create_anon_class(
                            roles => [qw/npg_common::roles::aligners/],
                        )->new_object();
             }
             'Create test object';

    is( $test->collection_root(), '/software/solexa/bin/aligners',
        'Return default base directory of collection' );

    is( $test->collection_bin(), '/software/solexa/bin/aligners/bin',
        'Return default bin directory of collection' );
}


# Test setting attributes
{
    lives_ok {
                $test = Moose::Meta::Class->create_anon_class(
                            roles => [
                                        qw/npg_common::roles::aligners/
                                     ],
                        )->new_object(
                            {
                                collection_root => 't/data/aligners',
                                collection_bin  => 't/data/aligners/bin',
                            }
                );
            }
            'Create test object with options';

    is( $test->collection_root(), 't/data/aligners',
        'Return specified base directory of collection' );

    is( $test->collection_bin(), 't/data/aligners/bin',
        'Return specified bin directory of collection' );

    cmp_bag( $test->aligner_list(),
             [ qw/ picard bowtie bwa samtools abyss fuzzypath / ],
             'The returned aligner list is correct' );
}


# Test the method that finds the current base subdirectory for an aligner
{
    throws_ok { $test->current_base_dir(); }
              qr/Aligner name required as argument/ms,
              'The aligner name must be passed';

    like( $test->current_base_dir('bowtie'), qr{t/data/aligners/bowtie/bowtie_2.4.6_test},
        'Return the correct current base dir' );

    like( $test->current_base_dir('abyss'),
        qr{t/data/aligners/abyss/1.2.3/installed/bin},
        'Abyss is a special case' );

    warning_is { $test->current_base_dir('bwa') }
               { carped => 't/data/aligners/bwa/current not found' },
               'The current directory does not exist';
}


# Test the method that returns the version string for the current version
# The 'aligners' are just bash scripts that simulate the version reporting
# behaviour of the real aligners.
{
    throws_ok { $test->current_version(); }
              qr/Aligner name required as argument/ms,
              'Require aligner name for version method';

    is ( $test->current_version('bwa'),       q{0.5.8c (r1536)},
         'Return version string for bwa' );
    is ( $test->current_version('bowtie'),    q{0.12.1},
         'Return version string for bowtie' );
    is ( $test->current_version('maq'),       q{0.7.1},
         'Return version string for maq' );
    is ( $test->current_version('samtools'),  q{0.1.11 (r851)},
         'Return version string for samtools' );
    is ( $test->current_version('velvet'),    q{1.0.13},
         'Return version string for velvet' );
    is ( $test->current_version('stampy'),    q{v1.0.11 (r880)},
         'Return version string for stampy' );
    is ( $test->current_version('smalt'),     q{0.4},
         'Return version string for smalt' );
    is ( $test->current_version('soap'),      q{2.20},
         'Return version string for soap2' );
    is ( $test->current_version('ssaha2'),    q{2.5},
         'Return version string for ssaha2' );
    is ( $test->current_version('abyss'),     q{1.2.1},
         'Return version string for abyss' );
    is ( $test->current_version('fuzzypath'), q{fuzzypath_v3.1},
         'Return version string for fuzzypath' );
    is ( $test->current_version('picard'),    q{picard-tools-1.36},
         'Return version string for picard' );
}


# Test the method that lists all available versions of an aligner
{
    throws_ok { $test->installed_versions(); }
              qr/Aligner name required as argument/ms,
              'Require aligner name for version listing';

    throws_ok { $test->installed_versions('No_Such'); }
              qr{No_Such not maintained in t/data/aligners}ms,
              'Croak if we\'ve never heard of the aligner';

    my $empty;
    warning_is { $empty = $test->installed_versions('bwa'); }
               {
                 carped =>
                  'Could not establish the list of installed versions for bwa'
               },
               'Warn that no installed versions were found';

    is_deeply( $empty, [], 'But return an empty array-ref' );

    cmp_bag(
        $test->installed_versions('bowtie'),
        [ qw/ 2.4.6_test 2.4.5 1.3.3.3 / ],
        'Find all installed versions of bowtie'
    );
}

1;
