use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 26;
use Test::Exception;
use lib qw{t};

BEGIN {
  use_ok( q{npg_common::roles::test_type_of_value} );
}

{

  my $type_tester;
  lives_ok {
    $type_tester = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::roles::test_type_of_value/]
    )->new_object();
  } q{create object with role in it};

  my $array = [];
  my $code = sub { return 1; };
  open my $fh, q{<}, q{t/40-roles_test_type_of_value.t} or die;
  my $hash = {};
  my $int = 45;
  my $num = q{-12.07};
  my $regexp = qr/hello[ ]world/;
  my $string = q{scalar value};
  my $scalar_ref = \$string;

  ok( $type_tester->is_array( $array ), q{is array} );
  ok( !$type_tester->is_array( $code ), q{not array} );
  ok( $type_tester->is_code( $code ), q{is code} );
  ok( !$type_tester->is_code( $array ), q{not code} );
  ok( $type_tester->is_filehandle( $fh ), q{is filehandle} );
  ok( !$type_tester->is_filehandle( $array ), q{not filehandle} );
  ok( $type_tester->is_glob( $fh ), q{is glob} );
  ok( !$type_tester->is_glob( $array ), q{not glob} );
  ok( $type_tester->is_hash( $hash ), q{is hash} );
  ok( !$type_tester->is_hash( $array ), q{not hash} );
  ok( $type_tester->is_int( $int ), q{is int} );
  ok( !$type_tester->is_int( $num ), q{not int} );
  ok( $type_tester->is_num( $num ), q{is num} );
  ok( !$type_tester->is_num( $string ), q{not num} );
  ok( $type_tester->is_object( $type_tester ), q{is object} );
  ok( !$type_tester->is_object( $hash ), q{not object} );
  ok( $type_tester->is_regexp( $regexp ), q{is regexp} );
  ok( !$type_tester->is_regexp( $hash ), q{not regexp} );
  ok( $type_tester->is_scalar( $string ), q{is scalar (value)} );
  ok( !$type_tester->is_scalar( $hash ), q{not scalar (value)} );
  ok( $type_tester->is_scalar_ref( $scalar_ref ), q{is scalar_ref} );
  ok( !$type_tester->is_scalar_ref( $string ), q{not scalar_ref} );
  ok( $type_tester->is_string( $string ), q{is string} );
  ok( !$type_tester->is_string( $scalar_ref ), q{not string} );

}
1;