#############
# $Id: run_short_info.pm 13436 2011-06-23 11:24:01Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2009-10-01
# Last Changed On: $Date: 2011-06-23 12:24:01 +0100 (Thu, 23 Jun 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-43.0/t/lib/npg_common/role_tests/run_short_info.pm $

package npg_common::role_tests::run_short_info;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use File::Spec::Functions qw(splitdir);
use List::Util qw(first);


Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13436 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar my $FOLDER_PATH_PREFIX_GLOB_PATTERN => '/{staging,nfs/sf{5,6,9,10,11,12,13,14,15,16,17}}/{IL,HS}*/*/';

with qw{npg_common::roles::run::short_info};

sub _build_run_folder {
  my ($self) = @_;

  if( !( $self->has_id_run() || $self->has_name() ) ) {
    croak q{Need an id_run or name to generate the run_folder};
  }

  my $path = $self->_short_path();
  return first {$_ ne q()} reverse splitdir($path);
  
}


has q{_short_path}                => ( isa => q{Str}, is => q{ro}, lazy_build => 1, init_arg => undef );

sub _build__short_path {
  my ($self) = @_;
  my @dir = $self->has_run_folder() ? glob $self->_folder_path_glob_pattern() . $self->run_folder()
          : $self->has_id_run()     ? glob $self->_folder_path_glob_pattern() . q(*_{r,}) . $self->id_run() . q{*} 
          : $self->has_name()       ? glob $self->_folder_path_glob_pattern() . q{*} . $self->name()
          :                           croak q{No run_folder, name or id_run provided}
          ;

  @dir = grep {-d $_} @dir;

  if ( @dir == 0 ) {
    croak 'No paths to run folder found.';
  }

  my %fs_inode_hash; #ignore multiple paths point to the same folder
  @dir = grep {not $fs_inode_hash{join q(,),stat $_}++} @dir;

  if ( @dir > 1 ) {
    croak 'Ambiguous paths for run folder found: '.join q{,},@dir;
  }
  return shift @dir;
}

has 'pattern_prefix' => (
  is => 'ro',
  isa => 'Str',
  default => q[/nonexisting],
);

sub _folder_path_glob_pattern {
  my $self = shift;
  return $self->pattern_prefix . $FOLDER_PATH_PREFIX_GLOB_PATTERN;
}



no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_common::role_tests::run_short_info

=head1 VERSION

$LastChangedRevision: 13436 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This is a test class object for testing the role npg_common::roles::run::short_info, whilst you may want to use
some of this code, it should not be considered production

=head1 SUBROUTINES/METHODS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
